import logging
from typing import Dict, Any, List, Optional
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase
from src.config.config import Config
from src.utils.token_display_formatter import get_formatter

logger = logging.getLogger(__name__)

class MarketCommands(InfoCommandsBase):
    """Handles all market-related commands."""

    def __init__(self, trading_engine, notification_manager):
        super().__init__(trading_engine, notification_manager)
        self.formatter = get_formatter()

    async def market_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /market command to show market information."""
        try:
            if not self._is_authorized(update):
                await self._reply(update, "❌ Unauthorized access.")
                return

            # Get token from command arguments or use default
            token = context.args[0].upper() if context.args else Config.DEFAULT_TRADING_TOKEN
            
            # Construct the full market symbol
            symbol = f"{token}/USDC:USDC"
            
            # Get market data
            market_data = await self.trading_engine.get_market_data(symbol)
            if not market_data:
                await self._reply(update, f"❌ Could not get market data for {symbol}. Check if the symbol is correct.")
                return
            
            # Get all markets to find leverage info
            all_markets = await self.trading_engine.client.get_markets()
            market_info = all_markets.get(symbol) if all_markets else None
            
            # Extract ticker and orderbook for easier access
            ticker = market_data.get('ticker', {})
            orderbook = market_data.get('orderbook', {})
            
            # Safely get nested info dictionary
            ticker_info = ticker.get('info', {})

            # Calculate 24h change
            last_price = ticker.get('last', 0)
            prev_close = ticker.get('previousClose', 0)
            change_24h = ((last_price - prev_close) / prev_close * 100) if prev_close else 0

            # Format the message
            message = f"📊 <b>Market Information for {token}</b>\n\n"
            
            # Add price information
            message += "💰 <b>Price Information:</b>\n"
            message += f"Last Price: {self.formatter.format_price_with_symbol(last_price)}\n"
            message += f"24h High: {self.formatter.format_price_with_symbol(ticker.get('high', 0) or 0)}\n"
            message += f"24h Low: {self.formatter.format_price_with_symbol(ticker.get('low', 0) or 0)}\n"
            message += f"24h Change: {change_24h:.2f}%\n"
            
            # Add leverage information if available
            if market_info:
                max_leverage = ticker_info.get('maxLeverage', 'N/A')
                if max_leverage != 'N/A':
                    message += f"Leverage: Up to {max_leverage}x\n"
            
            # Add volume information
            message += f"\n📈 <b>Volume Information:</b>\n"
            message += f"24h Volume: {self.formatter.format_amount(float(ticker_info.get('dayBaseVlm', 0)), token)}\n"
            message += f"24h Turnover: {self.formatter.format_price_with_symbol(float(ticker_info.get('dayNtlVlm', 0)))}\n"
            
            # Add order book information if available
            if orderbook:
                spread = orderbook.get('ask', 0) - orderbook.get('bid', 0)
                message += f"\n📚 <b>Order Book:</b>\n"
                message += f"Bid: {self.formatter.format_price_with_symbol(orderbook.get('bid', 0))}\n"
                message += f"Ask: {self.formatter.format_price_with_symbol(orderbook.get('ask', 0))}\n"
                message += f"Spread: {self.formatter.format_price_with_symbol(spread)}\n"
            
            await self._reply(update, message.strip())
            
        except Exception as e:
            logger.error(f"Error in market command: {e}")
            await self._reply(update, "❌ Error getting market information. Please try again later.")
